@@ -2,11 +2,15 @@
 #define PBC_H
 
 #include <stdio.h>
+#include <stdlib.h>
 #include <gmp.h>
 #include "fops.h"
 #include "symtab.h"
+#include "darray.h"
 #include "field.h"
 #include "pairing.h"
+#include "poly.h"
+#include "curve.h"
 #include "mnt.h"
 
 #endif //PBC_H
